@@ -358,6 +358,10 @@ def _postprocess_xml(context):
 	assert context is not None
 	assert isinstance(context, project.Context)
 
+	# delete the new Doxyfile.xml (https://github.com/doxygen/doxygen/pull/8463)
+	# (it breaks m.css otherwise)
+	delete_file(Path(context.xml_dir, r'Doxyfile.xml'), logger=context.verbose_logger)
+
 	xml_files = get_all_files(context.xml_dir, any=(r'*.xml'))
 	if not xml_files:
 		return
@@ -479,10 +483,24 @@ def _postprocess_xml(context):
 						for typedef in typedefs:
 							cpp_tree.add_type(rf'{scope_name}::{typedef.find("name").text}')
 
+					# enumerate all compound pages and their types for use later in the HTML post-process
+					pages = {}
+					for tag in root.findall(r'compound'):
+						refid = tag.get(r'refid')
+						filename = refid
+						if filename == r'indexpage':
+							filename = r'index'
+						filename = filename + r'.html'
+						pages[filename] = { r'kind' : tag.get(r'kind'), r'name' : tag.find(r'name').text, r'refid' : refid }
+					context.__dict__[r'compound_pages'] = pages
+					context.verbose_value(r'Context.compound_pages', pages)
+
 				# some other compound definition
 				else:
 					compounddef = root.find(r'compounddef')
-					assert compounddef is not None
+					if compounddef is None:
+						context.warning(rf'{xml_file} did not contain a <compounddef>!')
+						continue
 					compoundname = compounddef.find(r'compoundname')
 					assert compoundname is not None
 					assert compoundname.text
@@ -619,6 +637,19 @@ def _postprocess_xml(context):
 										compounddef.remove(tag)
 										changed = True
 
+					# groups and namespaces
+					if compounddef.get(r'kind') in (r'group', r'namespace'):
+
+						# fix inner(class|namespace|group) sorting
+						inners = [tag for tag in compounddef.iterchildren() if tag.tag.startswith(r'inner')]
+						if inners:
+							changed = True
+							for tag in inners:
+								compounddef.remove(tag)
+							inners.sort(key=lambda tag: tag.text)
+							for tag in inners:
+								compounddef.append(tag)
+
 				if changed:
 					write_xml_to_file(xml, xml_file)
 
@@ -723,16 +754,31 @@ def _postprocess_html_file(path, context=None):
 	assert context is not None
 	assert isinstance(context, project.Context)
 
+	html_fixers = [f for f in context.fixers if isinstance(f, fixers.HTMLFixer)]
+	plain_text_fixers = [f for f in context.fixers if isinstance(f, fixers.PlainTextFixer)]
+
 	context.verbose(rf'Post-processing {path}')
-	changed = False
-	doc = soup.HTMLDocument(path, logger=context.verbose_logger)
-	for fix in context.fixers:
-		if fix(doc, context):
-			doc.smooth()
-			changed = True
-	if changed:
-		doc.flush()
-	return changed
+	html_changed = False
+	if html_fixers:
+		doc = soup.HTMLDocument(path, logger=context.verbose_logger)
+		for fix in html_fixers:
+			if fix(doc, context):
+				doc.smooth()
+				html_changed = True
+		if html_changed:
+			doc.flush()
+
+	plain_text_changed = False
+	if plain_text_fixers:
+		doc = [ read_all_text_from_file(path, logger=context.verbose_logger) ]
+		for fix in plain_text_fixers:
+			if fix(doc, context):
+				plain_text_changed = True
+		if plain_text_changed:
+			with open(path, 'w', encoding='utf-8', newline='\n') as f:
+				f.write(doc[0])
+
+	return html_changed or plain_text_changed
 
 
 
@@ -759,6 +805,7 @@ def _postprocess_html(context):
 			, fixers.CustomTags()
 			, fixers.EmptyTags()
 			, fixers.HeadTags()
+			, fixers.ImplementationDetails()
 		)
 		context.verbose(rf'Post-processing {len(files)} HTML files...')
 		if threads > 1:
@@ -947,8 +994,6 @@ def run(config_path='.',
 								if context.warnings.enabled:
 									warnings = _extract_warnings(outputs)
 									for w in warnings:
-										if context.warnings.treat_as_errors:
-											raise Exception(rf'{w} (warning treated as error)')
 										context.warning(w)
 
 						# remove the local paths from the tagfile since they're meaningless (and a privacy breach)
@@ -1000,8 +1045,6 @@ def run(config_path='.',
 								if context.warnings.enabled:
 									warnings = _extract_warnings(outputs)
 									for w in warnings:
-										if context.warnings.treat_as_errors:
-											raise Exception(rf'{w} (warning treated as error)')
 										context.warning(w)
 
 				# copy extra_files
@@ -1114,8 +1157,14 @@ def main():
 			sys.exit(0)
 		else:
 			sys.exit(1)
+	except WarningTreatedAsError as err:
+		print(rf'Error: {err} (warning treated as error)', file=sys.stderr)
+		sys.exit(1)
+	except Error as err:
+		print(rf'Error: {err}', file=sys.stderr)
+		sys.exit(1)
 	except Exception as err:
-		print_exception(err, include_type=verbose, include_traceback=verbose, skip_frames=1)
+		print_exception(err, include_type=True, include_traceback=True, skip_frames=1)
 		sys.exit(-1)
 
 
